
(defun underflow? (stack args message)
  (cond ((< (length stack) args)
	 (format t "~%UNDERFLOW ~A" message)
	 t)
	(t nil)))

(defun humanize (exp &optional (stack nil)) 
  ;; assumed: just atoms, all float, unary or binary (aside from stack ops) limited ops etc
  ;; unaries come out in prefix, binaries in infix
  (let ((stackops NIL) 
	(binaries '(*))
	(unaries NIL) 
	(vars '(X Y Z)))
    ;(format t "~%Stack depth is ~A before executing ~A." (length stack) (first exp))
    (cond ((null exp)
	   (if (underflow? stack 1 '(NO RESULT))
	       nil
	     (first stack)))
	  ((numberp (first exp))
	   (humanize (rest exp) (cons (first exp) stack)))
	  ((member (first exp) vars)
	   (humanize (rest exp) (cons (first exp) stack)))
	  ((member (first exp) stackops)
	   (case (first exp)
		 (FLOAT.DUP (if (underflow? stack 1 exp)
				(humanize (rest exp) stack)
			      (humanize (rest exp) (cons (first stack) stack))))
		 (FLOAT.POP (if (underflow? stack 1 exp)
				(humanize (rest exp) stack)
			      (humanize (rest exp) (rest stack))))
		 (FLOAT.ROT (if (underflow? stack 3 exp)
				(humanize (rest exp) stack)
			      (humanize (rest exp) 
					(cons (third stack)
					      (cons (first stack)
						    (cons (second stack)
							  (rest (rest (rest stack)))))))))
		 (FLOAT.SWAP (if (underflow? stack 2 exp)
				 (humanize (rest exp) stack)
			       (humanize (rest exp)
					 (cons (second stack)
					       (cons (first stack)
						     (rest (rest stack)))))))))
	  ((member (first exp) binaries)
	   (if (underflow? stack 2 exp)
	       (humanize (rest exp) stack)
	     (humanize (rest exp) 
		       (cons (list (second stack) (first exp) (first stack))
			     (rest (rest stack))))))
	  ((member (first exp) unaries)
	   (if (underflow? stack 1 exp)
	       (humanize (rest exp) stack)
	     (humanize (rest exp) 
		       (cons (list (first exp) (first stack))
			     (rest stack)))))
	  (t (format t "~%UNRECOGNIZED ITEM: ~A" (first exp))
	     (humanize (rest exp) stack)))))

(defun symbol-swap (exp)
  (let ((pairs '((FLOAT.* *)
		 (FLOAT.+ +)
		 (FLOAT.- -)
		 (FLOAT.COS COS)
		 (FLOAT.MAX MAX)
		 (FLOAT.MIN MIN)
		 (FLOAT.SIN SIN)
		 (FLOAT.TAN TAN)
		 (FLOAT.PROTECTEDDIVIDE /)
		 (FLOAT.PROTECTEDMOD %)
		 (FLOAT.INPUT1 DPREC)
                 (FLOAT.INPUT2 DHFLR)
                 (FLOAT.INPUT3 DQFLR)
                 (FLOAT.INPUT4 DAVAP)
                 (FLOAT.INPUT5 DLAIT)
                 (FLOAT.INPUT6 DLAIU)
                 (FLOAT.INPUT7 DSNOWF)
                 (FLOAT.INPUT8 DROUGHL)
                 (FLOAT.INPUT9 DSURWIND)
                 (FLOAT.INPUT10 DALBEDO)
                 (FLOAT.INPUT11 DSMOIST)
                 (FLOAT.INPUT12 DTRANSP)
                 (FLOAT.INPUT13 ABSLAT)
		 )))
    (dolist (pair pairs)
      (setq exp (subst (second pair) (first pair) exp)))
    exp))
    
(defun prune-unnecessary-parentheses (exp)
  (cond ((null exp) exp)
	((not (consp exp)) exp)
	((and (null (rest exp))
	      (listp (first exp)))
	 (prune-unnecessary-parentheses (first exp)))
	(t (mapcar #'prune-unnecessary-parentheses exp))))

(defparameter *already-printed* nil)

(defun print-one (name program)
  (let ((infix (prune-unnecessary-parentheses (humanize program))))
    (cond ((member infix *already-printed* :test #'equalp)
	   (format t "~%[duplicate not printed]~%~%"))
	  (t (push infix *already-printed*)
	     (print name)
	     (print infix)
	     (format t "~%Number of operations: ~A~%" (count #\* (format nil "~A" infix)))
	     (terpri)(terpri)))))

(defun r ()
    (print-one "compute-0-1.breve_cli.10-01_11/06/39.out"
	       '(Z X X * Y Y * X * X * X * * Y * Y * * X X * Y Y * X * X * X * * Y * X * *))
    (values)
)